/*

    This code is part of Truss Solver which is free software.
    See the file COPYING for more details.

    Copyright (C) 1997  T. W. Shield, shield@aem.umn.edu

*/
#include "gfprofile.h"
#include "gfcommon.h"
#include "gfutil.h"


GfProfile::GfProfile()
{
	init();
		
}

void GfProfile::init()
{
	use_eof = EOF_AUTO; /* use eof on reads */
	use_delim = FMT_FIXED;     /* use delimeter charater on reads 1==yes */
	delim_char = '\t';    /* delimeter character */
	dlen = 10;         /* the data length and invec length formats */
	vlen = 3; // note that now have invecs in delim data, set 
		// vlen = 0 to turn off invec with delims.
		
}

GfProfile::GfProfile(const GfProfile &source) // copy constructor
{
	use_eof=source.eof();
	use_delim=source.lineFormat();
	delim_char = source.delimChar();
	dlen = source.dataLength();
	vlen = source.invecLength();
	
}

GfProfile::GfProfile(const GfProfile *source) // copy constructor from pointer
{
	use_eof=source->eof();
	use_delim=source->lineFormat();
	delim_char = source->delimChar();
	dlen = source->dataLength();
	vlen = source->invecLength();
}

GfProfile::~GfProfile()
{
	
}

void GfProfile::defaults()
{
	init();
}


void GfProfile::print(OSTREAM *os)
{
	*os << PROFILEMAGIC <<"\n";

	*os << "C\tuseeof: "<< use_eof <<"\n";
	*os << "C\tusedelim: "<< use_delim <<"\n";
	*os << "C\tdelimchar: "<< (int)delim_char <<"\n";
	
	// hack for 1.234E+000 windows format problem
	// added winFixExpFmt, so don't need this anymore
	
//ifdef UNIX
	*os << "C\tdatalen: "<< dlen <<"\n";
//else
//	*os << "C\tdatalen: "<< dlen + 1 <<"\n";
//endif

	*os << "C\tinveclen: "<< vlen <<"\n";
}

int GfProfile::readFormat(QFile *in,char *line)
{
int t;

	if(strstr(line,PROFILEMAGIC) == NULL)
		return(GF_BAD);

	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("useeof",line,&use_eof) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}
	
	if(getintval("usedelim",line,&use_delim) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}
	
	// delim char is stored as an interger ASCII value
	if(getintval("delimchar",line,&t) != GF_OK)
		return(GF_BAD);
	delim_char = (char)t;
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("datalen",line,&dlen) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}

	if(getintval("inveclen",line,&vlen) != GF_OK)
		return(GF_BAD);
		
	in->readLine(line,MAX_LINE_LEN-1);
	if(in->atEnd())
	{
		return(GF_BAD);
	}
	
	return(GF_OK);

}

OSTREAM &operator<<(OSTREAM &os, GfProfile *m)
{
	m->print(&os);
	return(os);
}
OSTREAM &operator<<(OSTREAM &os, GfProfile &m)
{
	m.print(&os);
	return(os);
}

void GfProfile::printScale(OSTREAM *os,GfScale *xyscale)
{
char line[100],temp[100],fmt[20],*xlabel,*ylabel,ticks;
	
        if(use_delim == FMT_FIXED)
        {
		sprintf(fmt,"%%+%i.%iE",dlen,dlen-7);

		sprintf(temp,fmt,xyscale->xMin());
		strcpy(line,temp);

		sprintf(temp,fmt,xyscale->xMax());
		strcat(line,temp);
		
         	if(xyscale->xLogScale() == GF_YES) 
		{
			strcat(line,"L");
		}
		
		strcat(line,"\n");
				
		*os << line;

		sprintf(temp,fmt,xyscale->yMin());
		strcpy(line,temp);

		sprintf(temp,fmt,xyscale->yMax());
		strcat(line,temp);
		
         	if(xyscale->yLogScale() == GF_YES) 
		{
			strcat(line,"L");
		}
 
		strcat(line,"\n");
				
 		*os << line;

		if(xyscale->xLabelLen() > 0)
		{
			strmalloc(&xlabel,xyscale->xLabelLen());
			xyscale->xLabel(xlabel);
			*os <<xlabel<<"\n";
			free(xlabel);
		}
		else
			*os <<"\n";
		
		if(xyscale->yLabelLen() > 0)
		{
			strmalloc(&ylabel,xyscale->yLabelLen());
			xyscale->yLabel(ylabel);
			*os <<ylabel<<"\n";
			free(ylabel);
		}
		else
			*os<<"\n";

		switch(xyscale->ticksType())
		{
			case TT_TICKS:
				ticks='t';
			break;
			case TT_GRIDS:
				ticks='g';
			break;
			default:
				ticks='n';
			break;
		}
	
		sprintf(line,"%c",ticks);
		
		sprintf(temp,fmt,xyscale->xTicksStep());
		strcat(line,temp);

		sprintf(temp,fmt,xyscale->yTicksStep());
		strcat(line,temp);				
		
		strcat(line,"\n");
				
		*os << line;
	} // FMT_FIXED
	else
       {
        	if(xyscale->xLogScale() == GF_YES)
        	{
                	sprintf(fmt,"%%g%c%%gL\n",delim_char);
        	}
        	else
        	{
                	sprintf(fmt,"%%g%c%%g\n",delim_char);
        	}

        	sprintf(line,fmt,xyscale->xMin(),xyscale->xMax());
		*os << line;

        	if(xyscale->yLogScale() == GF_YES)
        	{
                	sprintf(fmt,"%%g%c%%gL\n",delim_char);
        	}
        	else
        	{
                	sprintf(fmt,"%%g%c%%g\n",delim_char);
        	}

        	sprintf(line,fmt,xyscale->yMin(),xyscale->yMax());
		*os << line;

	
		if(xyscale->xLabelLen() > 0)
		{
			strmalloc(&xlabel,xyscale->xLabelLen());
			xyscale->xLabel(xlabel);
			*os <<xlabel<<"\n";
			free(xlabel);
		}
		else
			*os <<"\n";
		
		if(xyscale->yLabelLen() > 0)
		{
			strmalloc(&ylabel,xyscale->yLabelLen());
			xyscale->yLabel(ylabel);
			*os <<ylabel<<"\n";
			free(ylabel);
		}
		else
			*os<<"\n";

		switch(xyscale->ticksType())
		{
			case TT_TICKS:
				ticks='t';
			break;
			case TT_GRIDS:
				ticks='g';
			break;
			default:
				ticks='n';
			break;
		}
	
                sprintf(fmt,"%%c%%g%c%%g\n",delim_char);
			
        	sprintf(line,fmt,ticks,xyscale->xTicksStep(),xyscale->yTicksStep());
		*os << line;
	} // FMT_DELIM
}

int GfProfile::readScale(QFile *in,GfScale *xyscale)
{
char fmt[20],fmt2[20],ticks;
int iv;
float x,y;
char line[MAX_LINE_LEN];

	if(use_delim == FMT_FIXED)
	{
        	sprintf(fmt,"%%%ig%%%ig",dlen,dlen);

		in->readLine(line,MAX_LINE_LEN-1);
		parse_data_fixed(line,&iv,&x,&y,1,2,0);

                xyscale->setXMaxMin(y,x);

                if(strlen(line) > (unsigned int)(2*dlen+1))
                {
                        if(line[strlen(line)-2] == 'L') xyscale->setXLogScale(GF_YES);
                }

		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);

                parse_data_fixed(line,&iv,&x,&y,1,2,0);

                xyscale->setYMaxMin(y,x);

                if(strlen(line) > (unsigned int)(2*dlen+1))
                {
                       	if(line[strlen(line)-2] == 'L') xyscale->setYLogScale(GF_YES);
                }

		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);
                strip_crlf(line);
		xyscale->setXLabel(line);
                
		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);
                strip_crlf(line);
		xyscale->setYLabel(line);

                strcpy(fmt2,"%c");
                strcat(fmt2,fmt);

		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);
			
                sscanf(line,fmt2,&ticks,&x,&y);

		if(ticks == 't' || ticks == 'T')
				xyscale->setTicksType(TT_TICKS);
		if(ticks == 'g' || ticks == 'G')
				xyscale->setTicksType(TT_GRIDS);

                xyscale->setYTicksStep(y);
                xyscale->setXTicksStep(x);

	} // FMT_FIXED read scaling
	else
	{
		in->readLine(line,MAX_LINE_LEN-1);
		
		sprintf(fmt,"%%g%c%%g",delim_char);

		strip_crlf(line);
		
                if(line[strlen(line)-1] == 'L') 
		{
			xyscale->setXLogScale(GF_YES);
			line[strlen(line)-1] = '\0';
		}
		sscanf(line,fmt,&x,&y);
                xyscale->setXMaxMin(y,x);
		
		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);

		strip_crlf(line);
		
                if(line[strlen(line)-1] == 'L') 
		{
			xyscale->setXLogScale(GF_YES);
			line[strlen(line)-1] = '\0';
		}
		sscanf(line,fmt,&x,&y);
                xyscale->setYMaxMin(y,x);

		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);
                strip_crlf(line);
		xyscale->setXLabel(line);
                
		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);
                strip_crlf(line);
		xyscale->setYLabel(line);

                strcpy(fmt2,"%c");
                strcat(fmt2,fmt);

		if(in->atEnd()) return(0);
                in->readLine(line,MAX_LINE_LEN-1);
			
                sscanf(line,fmt2,&ticks,&x,&y);

		if(ticks == 't' || ticks == 'T')
				xyscale->setTicksType(TT_TICKS);
		if(ticks == 'g' || ticks == 'G')
				xyscale->setTicksType(TT_GRIDS);

                xyscale->setYTicksStep(y);
                xyscale->setXTicksStep(x);

	} // FMT_DELIM read scaling
	
	return(1);
}

int GfProfile::parse_data_fixed(char *line,int *i1, float *x, 
	float *y,int xcol,int ycol,int vlen2)
{
char part[50],*temp;

        if( vlen2 > 0 )
        {
		strncpy(part,line,vlen2);
                part[vlen2]='\0';
                sscanf(part,"%i",i1);
        }
        else
        {
                *i1=1;
        }

        temp=&line[vlen2+(xcol-1)*dlen];

 	strncpy(part,temp,dlen);
 	part[dlen]='\0';
        sscanf(part,"%g",x);

        temp=&line[vlen2+(ycol-1)*dlen];

 	strncpy(part,temp,dlen);
	part[dlen]='\0';
        sscanf(part,"%g",y);
	
#ifdef DEBUG1
	cout <<"pdf: "<<*x<<" "<<*y<<"\n";
#endif	

        return(GF_OK); /* every thing ok */
}

void GfProfile::strip_crlf(char *text)
{
/*  
	strips any trailing \r or \n  from text 
*/
int len;
	len = strlen(text);
	
	if(len < 1) return; /* empty string */
	
	len=len-1;
	
	while(text[len] == '\r' || text[len] == '\n')
	{
		text[len]='\0'; /* null it out */
		len--;
		if(len < 0) return;
	}
}


void GfProfile::operator=(const GfProfile &source)
{
	use_eof=source.eof();
	use_delim=source.lineFormat();
	delim_char = source.delimChar();
	dlen = source.dataLength();
	vlen = source.invecLength();

}


int GfProfile::setEof(int t)
{
	switch(t)
	{
		case EOF_AUTO:
			use_eof = t;
			return(GF_OK);
		case EOF_YES:
			use_eof = t;
			return(GF_OK);
		case EOF_NO:
			use_eof = t;
			return(GF_OK);
		default:
			return(GF_BAD);
	}
}

int GfProfile::setLineFormat(int t)
{
	switch(t)
	{
		case FMT_FIXED:
			use_delim = t;
			return(GF_OK);
		case FMT_DELIM:
			use_delim = t;
			return(GF_OK);
		default:
			return(GF_BAD);
	}
}

int GfProfile::setDelimChar(char c)
{
	delim_char = c;
	return(GF_OK);
}
int GfProfile::setDataLength(int t)
{
	if( t > 7 )
	{
		dlen = t;
		return(GF_OK);
	}
	else
		return(GF_BAD);
}
int GfProfile::setInvecLength(int t)
{
	if( t >= 0 )
	{
		vlen = t;
		return(GF_OK);
	}
	else
		return(GF_BAD);
}
	
int GfProfile::eof() const 
{
	return(use_eof);
}
int GfProfile::lineFormat() const 
{
	return(use_delim);
}

char GfProfile::delimChar() const 
{
	return(delim_char);
}

int GfProfile::dataLength() const 
{
	return(dlen);
}

int GfProfile::invecLength() const 
{
	return(vlen);
}


